package sbc.mgclus.algorithm.scoring;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.jgrapht.WeightedGraph;
import org.jgrapht.graph.DefaultWeightedEdge;

import sbc.mgclus.algorithm.ClusteringKL;
import sbc.mgclus.data.Cluster;
import sbc.mgclus.data.Vertex;

public class CommonNeighborsS {
	private WeightedGraph<Cluster, DefaultWeightedEdge> graph;
	private Map<Integer, Double> sharedNeighbors;
	private Integer numberOfVertices;
	private Boolean useWeights;
	private Logger logger=Logger.getLogger(ClusteringKL.class);
	
	public CommonNeighborsS(WeightedGraph<Cluster, DefaultWeightedEdge> g, Boolean w){
		graph = g;
		useWeights = w;
		logger.info(String.format("find common neighbors...\n"));
		findCommonNeighbors();
		//logger.info(String.format("finished finding common neighbors...\n"));
		
	}
	
	private void findCommonNeighbors(){
		//initiate a HashMap to store for each pair of vertices the number of shared neighbors
		sharedNeighbors = new HashMap<Integer, Double>();
		numberOfVertices = (graph.vertexSet()).size();
		int nbKey;
		//iterate over each node in the graph
		for (Cluster c : graph.vertexSet()){
			//get all neighbors
			ArrayList<Cluster> neighbors = getNeighbors(c);
			for (int i=0; i < neighbors.size(); i++ ){
				Cluster neighbor1 = neighbors.get(i);
				for (int j=i+1; j < neighbors.size(); j++ ){
					Cluster neighbor2 = neighbors.get(j);
					if (neighbor1.getClusterID() > neighbor2.getClusterID())
						nbKey = (neighbor1.getClusterID()*numberOfVertices)+neighbor2.getClusterID();
					else
						nbKey = (neighbor2.getClusterID()*numberOfVertices)+neighbor1.getClusterID();
					if (!(sharedNeighbors.containsKey(nbKey)))
						sharedNeighbors.put(nbKey, 0.0);
					double cnScore = 0.0;
					if (useWeights)
						cnScore = (graph.getEdgeWeight(graph.getEdge(c, neighbor1))) * (graph.getEdgeWeight(graph.getEdge(c, neighbor2)));
					else
						cnScore = 1;
					sharedNeighbors.put(nbKey, ((sharedNeighbors.get(nbKey))+cnScore));
					/*logger.debug((c.getMembers()).get(0));
					logger.debug(graph.getEdgeWeight(graph.getEdge(c, neighbor1)));
					logger.debug(graph.getEdgeWeight(graph.getEdge(c, neighbor2)));
					logger.debug((neighbor1.getMembers()).get(0)+" "+(neighbor2.getMembers()).get(0)+" "+sharedNeighbors.get(nbKey));*/
				}
			}
		}
		/*for (Cluster c1 : graph.vertexSet()){
			Vertex v1 = (c1.getMembers()).get(0);
			for (Cluster c2 : graph.vertexSet()){
				if (c1.getClusterID()<c2.getClusterID()){
					Vertex v2 = (c2.getMembers()).get(0);
					double commonNB = getSharedNeighbors(v1,v2);
					logger.debug(v1.getName()+" "+v2.getName()+" "+commonNB);
				}
			}
		}*/
	}

	private ArrayList<Cluster> getNeighbors(Cluster c){
		ArrayList<Cluster> neighbors = new ArrayList<Cluster>();
		for (DefaultWeightedEdge e : graph.edgesOf(c)){
			Cluster sourceC = graph.getEdgeSource(e);
			Cluster targetC = graph.getEdgeTarget(e);
			if (!(sourceC.equals(c)))
				neighbors.add(sourceC);
			else
				neighbors.add(targetC);
		}
		return neighbors;
	}

	public double getSharedNeighbors(Vertex v, Vertex u){
		int nbKey;
		if (v.getID() > u.getID())
			nbKey = (v.getID()*numberOfVertices)+u.getID();
		else
			nbKey = (u.getID()*numberOfVertices)+v.getID();
		//logger.debug(nbKey +" " + u.getID() + " " + v.getID() );
		if (sharedNeighbors.containsKey(nbKey))
			return sharedNeighbors.get(nbKey);
		else
			return 0;
	}
}
