package sbc.mgclus.algorithm.scoring;

import java.util.PriorityQueue;

import org.jgrapht.WeightedGraph;
import org.jgrapht.graph.DefaultWeightedEdge;

import sbc.mgclus.data.Cluster;
import sbc.mgclus.data.ClusterPair;
import sbc.mgclus.data.Vertex;

public class ScoringSeeds implements Scoring{
	private double alpha = 0.1;
	private WeightedGraph<Cluster, DefaultWeightedEdge> graph;
	private CommonNeighborsS commonNeighbors;
	
	public ScoringSeeds(WeightedGraph<Cluster, DefaultWeightedEdge> g, CommonNeighborsS cNB){
		this.graph = g;
		this.commonNeighbors = cNB;
	}
	
	public double calcMergeGain(Cluster c1, Cluster c2){
		double ratioEdgesNodesC1 = ((c1.getDegree())/(Math.pow(c1.getClusterSize(), 2)));
		double ratioEdgesNodesC2 = ((c2.getDegree())/(Math.pow(c2.getClusterSize(), 2)));
		double ratioEdgesNodesMerged = ((c1.getDegree()+c2.getDegree())/(Math.pow(c1.getClusterSize()+c2.getClusterSize(), 2)));
		//get the merge gains of the single clusters
		double mgC1 = ratioEdgesNodesC1 * (Math.log((ratioEdgesNodesC1+alpha)/((c1.getClusterEfficiency()/(Math.pow(c1.getClusterSize(), 2)))+alpha)));
		double mgC2 = ratioEdgesNodesC2 * (Math.log((ratioEdgesNodesC2+alpha)/((c2.getClusterEfficiency()/(Math.pow(c2.getClusterSize(), 2)))+alpha)));
		//calculate the efficiency of the two merged clusters
		//for all vertices in the two clusters find the number of common neighbors
		double commonNBscore = calcCNScore(c1, c2);
		//System.out.println(String.format("\tcommonNeighbors Score: %f", commonNBscore));
		//System.out.println(String.format("\tEdge Score: %f", graph.getEdgeWeight(graph.getEdge(c1, c2))));
		//!!!the score is multiplied by two to account for both vertex pairs a-b and b-a
		double mergedEfficiency = graph.getEdgeWeight(graph.getEdge(c1, c2)) + commonNBscore;
		mergedEfficiency += c1.getClusterEfficiency();
		mergedEfficiency += c2.getClusterEfficiency();
		double mgMerged = ratioEdgesNodesMerged * (Math.log((ratioEdgesNodesMerged+alpha)/((mergedEfficiency/(Math.pow(c1.getClusterSize()+c2.getClusterSize(), 2)))+alpha)));
		/*System.out.println(String.format("merge gains for: c%s, c%s", c1.getClusterID(), c2.getClusterID()));
		System.out.println(String.format("\tc%s, %f, %f", c1.getClusterID(), c1.getClusterEfficiency(), mgC1));
		System.out.println(String.format("\tc%s, %f, %f", c2.getClusterID(), c2.getClusterEfficiency(), mgC2));
		System.out.println(String.format("\tpair, %f, %f", mergedEfficiency, mgMerged));
		System.out.println(String.format("\tmerge efficiency: %f", mgC1 + mgC2 - mgMerged));*/
		double seedFreqC1 = c1.getSeedCount()/c1.getClusterSize();
		double seedFreqC2 = c2.getSeedCount()/c2.getClusterSize();
		double freqScore = (seedFreqC1 + seedFreqC2) / 2;
		return (mgC1 + mgC2 - mgMerged) * freqScore;
	}

	public PriorityQueue<ClusterPair> getInitialClusterPairs() {
		//System.out.println(String.format("number of total nodes: %d, number of total edges: %f\n", nrNodesTotal, nrEdgesTotal));
		PriorityQueue<ClusterPair> clustersToMerge = new PriorityQueue<ClusterPair>();
		//instantiate a cluster pair for each edge in the graph
		for (DefaultWeightedEdge edge : graph.edgeSet()){
			Cluster c1 = graph.getEdgeSource(edge);
			Cluster c2 = graph.getEdgeTarget(edge);
			if (((c1.getMembers()).get(0).isSeed()) && (((c2.getMembers()).get(0)).isSeed())){
				double mergeGain = calcMergeGain(c1,c2);
				if (mergeGain >= 0){
					ClusterPair cPair = new ClusterPair(c1, c2, mergeGain);
					clustersToMerge.add(cPair);
					//double w = graph.getEdgeWeight(edge);
					//System.out.println(String.format("pair added between: c%d, c%d, edge weight %f, merge gain %f\n", c1.getClusterID(), c2.getClusterID(), w, mergeGain));
					//System.out.println(String.format("pair added between: c%d (%s), c%d (%s), edge weight %f, merge gain %f\n", c1.getClusterID(), (c1.getMembers()).get(0), c2.getClusterID(), (c2.getMembers()).get(0), w, mergeGain));	
				}
			}
		}
		return clustersToMerge;
	}

	public double calcCNScore(Cluster c1, Cluster c2) {
		double commonNBscore = 0.0;
		for (Vertex v : c1.getMembers()){
			for (Vertex u : c2.getMembers()){
				commonNBscore += (Math.sqrt(commonNeighbors.getSharedNeighbors(v, u)));
			}
		}
		return commonNBscore;
	}
	
	/*
	public double calcCNScore(Cluster c1, Cluster c2) {
		double commonNBscore = 0.0;
		for (Vertex v : c1.getMembers()){
			for (Vertex u : c2.getMembers()){
				if ((v.isSeed()) && (u.isSeed()))
					commonNBscore += (Math.sqrt(commonNeighbors.getSharedNeighbors(v, u)));
				else if ((v.isSeed()) || (u.isSeed()))
					commonNBscore += (0.5*(Math.sqrt(commonNeighbors.getSharedNeighbors(v, u))));
			}
		}
		return commonNBscore;
	}
	
	private void recalcEdgeWeights(){
		for (DefaultWeightedEdge edge : graph.edgeSet()){
			Cluster c1 = graph.getEdgeSource(edge);
			Cluster c2 = graph.getEdgeTarget(edge);
			if ( (!((c1.getMembers()).get(0).isSeed())) && (!(((c2.getMembers()).get(0)).isSeed())) ){
				graph.setEdgeWeight(edge, 0);
			}
			else if ( (((c1.getMembers()).get(0).isSeed())) && (!(((c2.getMembers()).get(0)).isSeed())) ){
				double newEdgeWeight = 0.5*graph.getEdgeWeight(edge);
				graph.setEdgeWeight(edge, newEdgeWeight);
			}
			else if ( (!((c1.getMembers()).get(0).isSeed())) && ((((c2.getMembers()).get(0)).isSeed())) ){
				double newEdgeWeight = 0.5*graph.getEdgeWeight(edge);
				graph.setEdgeWeight(edge, newEdgeWeight);
			}
			//System.out.println(String.format("new edge score for: %s, %s, edge weight %f\n", ((c1.getMembers()).get(0)).getName(), ((c2.getMembers()).get(0)).getName(), graph.getEdgeWeight(edge)));
		}
	}
	*/
}
